import matplotlib.pyplot as plt
import numpy as np
from scipy.stats import hypergeom, nchypergeom_wallenius

WIDTH = 6.225  # \textwidth in inches

# hypergeom.pmf(k, N, K, n)
# nchypergeom_wallenius.pmf(k, N, K, n, w)

plt.rcParams.update({'figure.dpi': 300})
plt.rcParams.update({'font.size': 10})
plt.rcParams.update({'font.family': 'serif'})
plt.rcParams.update({'text.usetex': True})
plt.rcParams.update({'text.latex.preamble': '\\usepackage{mathpazo}'})

N = 30
K = np.arange(N+1)
plt.figure(figsize=(WIDTH, WIDTH/2))
fmts = ["p-", "D-", "s-", "o-"]
plot_hypergeom = lambda n: plt.plot(K, hypergeom.pmf(n, N, K, n), fmts.pop(), mec="w", label="$n=%d$" % n)
plot_hypergeom(1)
plot_hypergeom(2)
plot_hypergeom(4)
plot_hypergeom(8)
plt.xlabel("\\# of colluding witnesses $K$")
plt.ylabel("Probability $P(k=n)$")
plt.legend()
plt.title("Backdating probability (Hypergeometric distribution, $N=%d$)" % N)
plt.tight_layout()
plt.savefig("backdating_probability_hypergeometric.png")

n = 8
plt.figure(figsize=(WIDTH, WIDTH/2))
fmts = ["p-", "D-", "s-", "o-"]
plot_wallenius = lambda omega: plt.plot(K, nchypergeom_wallenius.pmf(n, N, K, n, omega), fmts.pop(), mec="w", label="$\\omega=%d$" % omega)
plot_wallenius(1)
plot_wallenius(10)
plot_wallenius(100)
plot_wallenius(1000)
plt.xlabel("\\# of colluding witnesses $K$")
plt.ylabel("Probability $P(k=n)$")
plt.legend()
plt.title("Backdating probability (Wallenius' distribution $N=%d, n=%d$)" % (N, n))
plt.tight_layout()
plt.savefig("backdating_probability_noncentral.png")

plt.figure(figsize=(WIDTH, WIDTH/2))
fmts = ["p-", "D-", "s-", "o-"]
plot_hypergeom_avail = lambda nn: plt.plot(K, hypergeom.cdf(n, N, K, n) - hypergeom.cdf(nn - 1, N, K, n), fmts.pop(), mec="w", label="$n'=%d$" % nn)
plot_hypergeom_avail(8)
plot_hypergeom_avail(4)
plot_hypergeom_avail(2)
plot_hypergeom_avail(1)
plt.xlabel("\\# of colluding witnesses $K$")
plt.ylabel("Probability $P(k\geq n')$")
plt.legend()
plt.title("Backdating vs. availability (Hypergeometric distribution, $N=%d, n=%d$)" % (N, n))
plt.tight_layout()
plt.savefig("backdating_probability_hypergeometric_available.png")

omega = 10
plt.figure(figsize=(WIDTH, WIDTH/2))
fmts = ["p-", "D-", "s-", "o-"]
plot_wallenius_avail = lambda nn: plt.plot(K, nchypergeom_wallenius.cdf(n, N, K, n, omega) - nchypergeom_wallenius.cdf(nn - 1, N, K, n, omega), fmts.pop(), mec="w", label="$n'=%d$" % nn)
plot_wallenius_avail(8)
plot_wallenius_avail(6)
plot_wallenius_avail(4)
plot_wallenius_avail(1)
plt.xlabel("\\# of colluding witnesses $K$")
plt.ylabel("Probability $P(k\geq n')$")
plt.legend()
plt.title("Backdating vs. availability (Wallenius' distribution, $N=%d, n=%d, \\omega=%d$)" % (N, n, omega))
plt.tight_layout()
plt.savefig("backdating_probability_noncentral_available.png")
